# coding: utf-8

"""
PSI_Collector

Module for testing.   # noqa: E501
"""

from setuptools import setup, find_packages  # noqa: H301

NAME = "psi_collector"
VERSION = "0.7.1"
REQUIRES = ["boto3", "pytz", "requests", "typeguard"]

# To install the library, run the following
#
# python setup.py install 
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

setup(
    name=NAME,
    version=VERSION,
    description="PSI Collector module",
    author="Pearson Service Insights",
    author_email="TOIOServiceWorkflowAutomation@grp.pearson.com",
    url="https://github.com/orgs/pearson-digitalandtechnology/POE-python-packages",
    keywords=["Pearson", "PSI", "Collector", "PSI_Collector"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
     Module containing collectors that extract from apis.   # noqa: E501
    """,
)
